"""
Usage:
    script.py results [-s <tag>]... [-y <year>] [-y <year>] [options]
    script.py records [-s <tag>] [-s <tag>] [-y <year>] [-y <year>] [options]
    script.py settable [-s <tag>]... [-y <year>] [-y <year>] [options]
    script.py -h | --help

Get tournament results of specified smasher

Options:
  -h, --help                Show this help message and exit
  -s, --smasher <tag>       The tag of the smasher you want results for
  -i, --input_file <path>   Path to input file where tags are stored
  -o, --output_file <path>  Path to output file where results are put
  -t, --threshold <place>   Tournaments where the smasher placed worse will have
                            their names displayed
  -y, --year <year>         Specified year used to filter tournament dates
                            List 1 specific year or 2 to define a range
  -g, --game <game>         Specified game to get tournament results for
                            [default: Melee]
  -e, --event <event>       What event to pull results for
                            [default: Singles]
  --debug                   Run in debug mode
"""

from smasherstats import *
from prettytable import PrettyTable, ALL
import os.path

def nums_from_string(string):
    nums = ''
    for char in string:
        if char.isnumeric():
            nums += char
    return int(nums)

def output_write(output, path):
    if os.path.isfile(path):
        with open(path, 'a+', encoding='utf8') as f:
            ofile = path
            for c in ['\\', '/']:
                ofile = ofile.replace(c, ' ')
            ofile = ofile.split()[-1]
            if output not in open(path).read():
                f.write(output + '\n\n')
                print(f'Output written to {ofile}')
            else:
                print(f'Output already in {ofile}')
    else:
        print(output)


# globals
smasher = ''
tags = []
threshold = 1
year = [CURRENT_YEAR]
game = 'Melee'
results_game = 'Melee'
record_game = 'melee'
event = ''
input_file = ''
output_file = ''
valid = 0

args = docopt(__doc__)
if args['--debug']:
    print(args)
for arg in args:
    if args[arg] is not None:
        globals()[arg.strip('-')] = args[arg]

for y in year:
    if not str(y).isnumeric() and y.upper() != 'ALL':
        print('Invalid year \'{y}\'. Defaulting to current year.')
        y = CURRENT_YEAR
    y = str(y).lstrip('0')
    try:
        y = int(y)
    except:
        pass
if year == []:
    year = [CURRENT_YEAR]

games = [
    ['MELEE',
     'SMASH MELEE',
     'SMASH BROS MELEE',
     'Super Smash Bros. Melee',
     'melee'],

    ['64',
     'SMASH 64',
     'SUPER SMASH BROS 64',
     'Super Smash Bros.',
     'smash-64'],

    ['BRAWL',
     'SMASH BROS BRAWL',
     'SMASH BRAWL',
     'Super Smash Bros. Brawl',
     'brawl'],

    ['SMASH 4',
     'SM4SH',
     'SMASH WII U',
     'SMASH BROS 4',
     'SMASH BROS WII U',
     'SMASH BROS 4',
     'SUPER SMASH BROS 4',
     'Super Smash Bros. for Wii U',
     'wii-u'],

    ['PM',
     'PROJECT MELEE',
     'SUPER SMASH BROS PROJECT M',
     'SUPER SMASH BROS PM',
     'Project M',
     'pm']
]
for g in games:
    if game.strip('.').upper() in g:
        game = g[-2]
        record_game = g[-1]
        results_game = g[0]
        valid = 1
if not valid:
    print('Invalid game \'{game}\'. Defaulting to Melee.')
    game = 'Super Smash Bros. Melee'
    record_game = 'melee'

events = [
    ['SINGLES',
     'Singles'],

    ['DOUBLES',
     'Doubles']
]

for e in events:
    if event.upper() in e:
        event = e[-1]
        valid = 1
if not valid:
    print('Invalid event \'{event}\'. Defaulting to Singles.')
    event = 'Singles'

if input_file != '':
    tags = [line.strip('\n') for line in open(input_file, 'r')]
if tags == [] and smasher == []:
    smasher = [input('Smasher: ')]
for tag in smasher:
    if tag != '' and tag.lower() not in map(str.lower, tags):
        tags += [tag]

results = []
tags = [' '.join(i[0].upper() + i[1:] for i in tag.split()) for tag in tags]
for tag in tags:
    r = getResults(tag, year, game, event)
    results.append([r[2], r[0]])
    year = r[1]

if args['results']:
    for i in range(len(tags)):
        tag = results[i][0]
        res = results[i][1]
        output = '-'*20 + '\n'
        output += f'{tag}\'s {results_game.capitalize()} {event} results for '
        if len(year) == 1:
            output += str(year[0])
        elif len(year) == 2:
            output += f'<{year[0]}, {year[1]}>'
        output += ':'
        if int(threshold) not in [0, 1]:
            output += f'\nTournament names listed for placings of {threshold} or below.\n'

        res = [r for r in res if any(c.isdigit() for c in r[0])]
        res = sorted(res, key=lambda x: nums_from_string(x[0]))

        # sorted by place
        # formatted like so: [[place, name, year], ...]
        for j in range(len(res)):
            r = [j[0] for j in res]
            place = res[j][0]
            if res[j - 1][0] != place:
                output += '\n'
                count = r.count(place)
                t_str = f'{place} - {count}'
                if nums_from_string(place) >= int(threshold) > 0:
                    for k in range(len(res)):
                        if res[k][0] == place:
                            t_name = res[k][1]
                            t_year = str(res[k][2])
                            if t_str[0] != '\n':
                                t_str = '\n' + t_str
                            t_str += '\n - ' + t_name + ' '
                            if len(year) != 1 and t_year not in t_name:
                                t_str += t_year
                output += t_str
        if i == len(tags) - 1:
            output += '\n' + '-'*20
        output_write(output, output_file)

if args['records']:
    record = getRecord(tags, results, record_game)
    output = ''
    output += '\n'
    if len(tags) == 2 and output_file != '':
        output += f'{tags[0]} vs. {tags[1]}'
    output += '\n'
    if len(record[3]) > 0:
        output += 'Tournaments where specified players were present but results failed to be retrieved:'
        for f in record[3]:
          output += f'\n - {f}'
        output += '\n'

    pt = PrettyTable()
    if len(tags) == 1:
        pt.field_names = ['Tournament', 'Round', f'{tags[0]} vs. ↓', 'Score', 'Outcome']
    elif len(tags) == 2:
        pt.field_names = ['Tournament', 'Round', f'{tags[0]} - {tags[1]}', 'Winner']

    pt_rows = record[0]
    for i in range(len(pt_rows)):
        row = pt_rows[i]
        r_name = row[0]
        if i > 0:
            if r_name == pt_rows[i-1][0]:
                r_name = ''
            else:
                pt.add_row(['' for _ in range(len(pt.field_names))])
        pt.add_row([r_name] + row[1:])
    output += pt.get_string()

    if len(tags) == 2:
        output += f'\n\nTotal Set Count: {tags[0]} {record[1][0]} - {record[1][1]} {tags[1]}'
        output += f'\nTotal Game Count: {tags[0]} {record[2][0]} - {record[2][1]} {tags[1]}'
    output_write(output, output_file)

if args['settable']:
    settable = PrettyTable(hrules=ALL)
    settable.field_names = ['↓ vs. →'] + tags
    output = ''
    st = getSetTable(tags, results, record_game)
    for i in range(len(st[0])):
        settable.add_row([tags[i]] + st[0][i])
    output += '\n\n'
    if len(st[1]) > 0:
        output += 'Tournaments where specified players were present but results failed to be retrieved:'
        for f in st[1]:
          output += f'\n - {f}'
        output += '\n'
    output += settable.get_string()
    output_write(output, output_file)


import datetime
import sys
import re

import requests
from docopt import docopt
from bs4 import BeautifulSoup as bsoup
import pysmash

CURRENT_YEAR = datetime.datetime.now().year
smash = pysmash.SmashGG()

def getResults(tag, year, game, event):
    res = []
    smasher = '_'.join(i for i in tag.split())
    request = 'http://www.ssbwiki.com/Smasher:'
    page = requests.get(request + smasher)
    if page.status_code != 200:
        page = requests.get(request[:-8] + smasher)  # hehe
    soup = bsoup(page.content, "html.parser")
    while page.status_code != 200:
        print(f'Invalid tag \'{smasher}\'. Try again.')
        tag = input('Smasher: ')
        tag = ' '.join(i[0].upper() + i[1:] for i in tag.split())
        smasher = '_'.join(i for i in tag.split())
        page = requests.get(request + smasher)
        if page.status_code != 200:
            page = requests.get(request[:-8] + smasher)  # hehe
        soup = bsoup(page.content, "html.parser")

    tables = soup.find_all('div', {'id': 'mw-content-text'})[0].contents[2].contents[1].contents[1]
    for header in tables.find_all('h3'):
        if game in header.contents[0].text:
            tables = tables.contents[tables.index(header) + 2]
            break
    if str(year[0]).upper() == 'ALL':
        year = [int(tables.contents[3].contents[3].text.split(', ')[1]), CURRENT_YEAR]

    for i in range(3, len(tables.contents), 2):
        t = tables.contents[i]
        t_name = t.contents[1].text
        t_year = int((t.contents[3].text).strip(' ')[-4:])
        if event == 'Singles':
            t_place = str(t.contents[5].text).strip(' ')
        elif event == 'Doubles':
            t_place = str(t.contents[7].text).strip(' ')
        res += [[t_place, t_name, t_year]]
    res = [i for i in res if int(year[0]) <= i[2] <= int(year[-1])]
    return [res, year, tag]


def getRecord(tags, results, game):
    if len(tags) == 2:
        print(f'{tags[0]} vs. {tags[1]}')
    tournaments = [r[1] for res in results for r in res[1] if res[0] in tags]
    filter_t = []
    for t in tournaments:
        if tournaments.count(t) == len(tags) and t not in filter_t:
            filter_t.append(t)
    tournaments = filter_t

    setcounts = [0, 0]
    gamecounts = [0, 0]

    fail_tournaments = []
    records = []
    #print(tournaments)
    for tournament in tournaments:
        sys.stdout.write(f'Retrieving tournament {tournaments.index(tournament)+1}/{len(tournaments)}...')
        sys.stdout.write('\r')
        sys.stdout.flush()
        havePlayed = 0

        "TODO: Fix bug where latest bracket in tourney where both players where present is returned, \
               meaning that two players could have played in a previous bracket but still be present \
               in a later bracket (e.g. one in winners and one in losers). To fix, loop through all  \
               brackets and if both players are present, check that they actually played. If they    \
               played in losers, stop looking, but if they played in winners, keep looking until one \
               player gets knocked out of the tournament."
        try:
            tournament_name = '-'.join(re.sub(r'[^\w\s]','',tournament.replace('\'', ' ')).split())
            t = smash.tournament_show_event_brackets(tournament_name, f'{game}-singles')
        except:
            fail_tournaments.append(tournament)
            continue

        havePlayed = 0
        temp_tags = set(map(str.lower, tags))
        for bracket in list(reversed(t['bracket_ids'])):
            if not havePlayed:
                players = smash.bracket_show_players(bracket)
                if temp_tags.issubset([p['tag'].lower() for p in players]):
                    sets = smash.bracket_show_sets(bracket)
                    player_ids = ['' for _ in range(len(tags))]
                    for p in players:
                        if p['tag'] in tags:
                            player_ids[tags.index(p['tag'])] = p['entrant_id']
                    for s in sets:
                        if all(str(n) != 'None' for n in s.values()):
                            ids = [int(s['entrant_1_id']), int(s['entrant_2_id'])]
                            scores = [s['entrant_1_score'], s['entrant_2_score']]
                            if all(i in ids for i in player_ids):
                                havePlayed = 1
                                p_tag = ''
                                if len(tags) == 1:
                                    p_id = ids[not ids.index(player_ids[0])]
                                    for p in players:
                                        if p['entrant_id'] == p_id:
                                            p_tag = p['tag']
                                if ids[0] != player_ids[0]:
                                    ids.reverse()
                                    scores.reverse()
                                if scores[0] > scores[1]:
                                    outcome = 'WIN'
                                else:
                                    outcome = 'LOSS'
                                for i in range(len(gamecounts)):
                                    gamecounts[i] += scores[i]
                                setcounts[scores.index(max(scores))] += 1
                                if len(tags) == 1:
                                    res = [tournament,
                                           s['full_round_text'],
                                           p_tag,
                                           f'{scores[0]} - {scores[1]}',
                                           outcome]
                                elif len(tags) == 2:
                                    res = [tournament,
                                           s['full_round_text'],
                                           f'{scores[0]} - {scores[1]}',
                                           tags[player_ids.index(int(s['winner_id']))]]
                                records.append(res)
            else:
                break
    return records, setcounts, gamecounts, fail_tournaments

def getSetTable(tags, results, game):
    st = [['-' for _ in range(len(tags))] for _ in range(len(tags))]
    fail_tournaments = []
    for i in range(len(tags)):
        for j in range(i+1, len(tags)):
            record = getRecord([tags[i], tags[j]], results, game)
            print(record[3])
            for f in record[3]:
                if f not in fail_tournaments:
                    fail_tournaments.append(f)
            record = record[1]
            print('\n')
            st[i][j] = f'{record[0]} - {record[1]}'
            st[j][i] = f'{record[1]} - {record[0]}'
    return st, fail_tournaments


from distutils.core import setup
setup(
    name = 'smasherstats',
    packages = ['smasherstats'],
    version = '1.3.4',
    description = 'Library for stats gathering on smash players using web scrapers',
    author = 'Michael Krasnitski',
    author_email = 'michael.krasnitski@gmail.com',
    license='MIT',
    url = 'https://github.com/PolarBearITS/smasherstats',
    download_url = 'https://github.com/PolarBearITS/smasherstats/tarball/1.3.4',
    install_requires = [
        'requests',
        'docopt',
        'beautifulsoup4',
        'pysmash'
    ]
)